@@ -39,20 +39,24 @@ enum tree_code {
    `d' for a decl, `t' for a type, `s' for a statement,
    and `x' for anything else (TREE_LIST, IDENTIFIER, etc).  */
 
-extern char *tree_code_type[];
+extern char **tree_code_type;
 
 /* Number of argument-words in each kind of tree-node.  */
 
-extern int tree_code_length[];
+extern int *tree_code_length;
+
+/* Names of tree components.  */
+
+extern char **tree_code_name;
 
 /* Get the definition of `enum machine_mode' */
 
 #ifndef HAVE_MACHINE_MODES
-#define DEF_MACHMODE(SYM, NAME, TYPE, SIZE, UNIT)  SYM,
+#define DEF_MACHMODE(SYM, NAME, TYPE, SIZE, UNIT, WIDER)  SYM,
 
 enum machine_mode {
 #include "machmode.def"
-};
+MAX_MACHINE_MODE };
 
 #undef DEF_MACHMODE
 
@@ -88,6 +92,7 @@ enum built_in_function
   BUILT_IN_GETEXP,
   BUILT_IN_GETMAN,
   BUILT_IN_SAVEREGS,
+  BUILT_IN_CLASSIFY_TYPE,
 
   /* C++ extensions */
   BUILT_IN_NEW,
@@ -128,7 +133,7 @@ struct tree_common
   int uid;
   union tree_node *chain;
   union tree_node *type;
-  enum tree_code code : 8;
+  unsigned char code : 8;
 
   unsigned external_attr : 1;
   unsigned public_attr : 1;
@@ -150,11 +155,8 @@ struct tree_common
   unsigned lang_flag_2 : 1;
   unsigned lang_flag_3 : 1;
   unsigned lang_flag_4 : 1;
-
-  unsigned overloaded_attr : 1;
-  unsigned private_attr : 1;
-  unsigned protected_attr : 1;
-  /* There is room for one more attribute.  */
+  unsigned raises_attr : 1;
+  /* There is room for three more attributes.  */
 };
 
 /* Define accessors for the fields that all tree nodes have
@@ -166,15 +168,12 @@ struct tree_common
 
 /* The tree-code says what kind of node it is.
    Codes are defined in tree.def.  */
-#define TREE_CODE(NODE) ((NODE)->common.code)
-#define TREE_SET_CODE(NODE, VALUE) ((NODE)->common.code = (VALUE))
+#define TREE_CODE(NODE) ((enum tree_code) (NODE)->common.code)
+#define TREE_SET_CODE(NODE, VALUE) ((NODE)->common.code = (int) (VALUE))
 
 /* In all nodes that are expressions, this is the data type of the expression.
    In POINTER_TYPE nodes, this is the type that the pointer points to.
-   In ARRAY_TYPE nodes, this is the type of the elements.
-
-   In IDENTIFIER_NODE nodes, if the identifier names an aggregate typedef,
-   this is the TYPE_DECL that it names.  */
+   In ARRAY_TYPE nodes, this is the type of the elements.  */
 #define TREE_TYPE(NODE) ((NODE)->common.type)
 
 /* Nodes are chained together for many purposes.
@@ -253,11 +252,12 @@ struct tree_common
 #define TREE_LITERAL(NODE) ((NODE)->common.literal_attr)
 
 /* Nonzero in a ..._DECL means this variable is ref'd from a nested function.
-   Cannot happen in C because it does not allow nested functions, as of now.
-   For VAR_DECL nodes, PARM_DECL nodes, and
-   maybe FUNCTION_DECL or LABEL_DECL nodes.
+   For VAR_DECL nodes, PARM_DECL nodes, and FUNCTION_DECL nodes.
 
-   C++: set for _DECL and _TYPE nodes that exist in class scope.  */
+   For LABEL_DECL nodes, nonzero if nonlocal gotos to the label are permitted.
+
+   Also set in some languages for variables, etc., outside the normal
+   lexical scope, such as class instance variables.  */
 #define TREE_NONLOCAL(NODE) ((NODE)->common.nonlocal_attr)
 
 /* Nonzero means permanent node;
@@ -276,7 +276,9 @@ struct tree_common
 #define TREE_ASM_WRITTEN(NODE) ((NODE)->common.asm_written_attr)
 
 /* Nonzero in a FUNCTION_DECL means this function can be substituted
-   where it is called.  */
+   where it is called.
+   Nonzero in a VAR_DECL or PARM_DECL means this decl was made by inlining;
+   suppress any warnings about shadowing some other variable.  */
 #define TREE_INLINE(NODE) ((NODE)->common.inline_attr)
 
 /* Nonzero in a _DECL if the name is used in its scope.  */
@@ -286,6 +288,9 @@ struct tree_common
 #define TREE_LANG_FLAG_2(NODE) ((NODE)->common.lang_flag_2)
 #define TREE_LANG_FLAG_3(NODE) ((NODE)->common.lang_flag_3)
 #define TREE_LANG_FLAG_4(NODE) ((NODE)->common.lang_flag_4)
+
+/* Nonzero for an expression that might raise an exception.  */
+#define TREE_RAISES(NODE) ((NODE)->common.raises_attr)
 
 /* Define additional fields and accessors for nodes representing constants.  */
 
@@ -366,24 +371,12 @@ struct tree_complex
 
 #define IDENTIFIER_LENGTH(NODE) ((NODE)->identifier.length)
 #define IDENTIFIER_POINTER(NODE) ((NODE)->identifier.pointer)
-#define IDENTIFIER_GLOBAL_VALUE(NODE) ((NODE)->identifier.global_value)
-#define IDENTIFIER_CLASS_VALUE(NODE) ((NODE)->identifier.class_value)
-#define IDENTIFIER_LOCAL_VALUE(NODE) ((NODE)->identifier.local_value)
-#define IDENTIFIER_LABEL_VALUE(NODE) ((NODE)->identifier.label_value)
-#define IDENTIFIER_IMPLICIT_DECL(NODE) ((NODE)->identifier.implicit_decl)
-#define IDENTIFIER_ERROR_LOCUS(NODE) ((NODE)->identifier.error_locus)
 
 struct tree_identifier
 {
   char common[sizeof (struct tree_common)];
   int length;
   char *pointer;
-  union tree_node *global_value;
-  union tree_node *class_value;
-  union tree_node *local_value;
-  union tree_node *label_value;
-  union tree_node *implicit_decl;
-  union tree_node *error_locus;
 };
 
 /* In a TREE_LIST node.  */
@@ -397,6 +390,18 @@ struct tree_list
   union tree_node *value;
 };
 
+/* In a TREE_VEC node.  */
+#define TREE_VEC_LENGTH(NODE) ((NODE)->vec.length)
+#define TREE_VEC_ELT(NODE,I) ((NODE)->vec.a[I])
+#define TREE_VEC_END(NODE) (&((NODE)->vec.a[(NODE)->vec.length]))
+
+struct tree_vec
+{
+  char common[sizeof (struct tree_common)];
+  int length;
+  union tree_node *a[1];
+};
+
 /* Define fields and accessors for some nodes that represent expressions.  */
 
 /* In a SAVE_EXPR node.  */
@@ -445,7 +450,6 @@ struct tree_exp
 #define TYPE_MIN_VALUE(NODE) ((NODE)->type.sep)
 #define TYPE_MAX_VALUE(NODE) ((NODE)->type.max)
 #define TYPE_PRECISION(NODE) ((NODE)->type.sep_unit)
-#define TYPE_PARSE_INFO(NODE) ((NODE)->type.parse_info)
 #define TYPE_SYMTAB_ADDRESS(NODE) ((NODE)->type.symtab_address)
 #define TYPE_NAME(NODE) ((NODE)->type.name)
 #define TYPE_NEXT_VARIANT(NODE) ((NODE)->type.next_variant)
@@ -468,7 +472,6 @@ struct tree_type
 
   union tree_node *pointer_to;
   union tree_node *reference_to;
-  int parse_info;
   int symtab_address;
   union tree_node *name;
   union tree_node *max;
@@ -489,13 +492,10 @@ struct tree_type
 #define DECL_FUNCTION_CODE(NODE) ((enum built_in_function) (NODE)->decl.offset)
 #define DECL_SET_FUNCTION_CODE(NODE,VAL) ((NODE)->decl.offset = (int) (VAL))
 #define DECL_NAME(NODE) ((NODE)->decl.name)
-#define DECL_PRINTABLE_NAME(NODE) ((NODE)->decl.printable_name)
-#define DECL_RTL_NAME(NODE) ((NODE)->decl.rtl_name)
 #define DECL_CONTEXT(NODE) ((NODE)->decl.context)
 #define DECL_FIELD_CONTEXT(NODE) ((NODE)->decl.context)
 #define DECL_ARGUMENTS(NODE) ((NODE)->decl.arguments)  /* In FUNCTION_DECL.  */
-#define DECL_ARG_TYPE(NODE) ((NODE)->decl.arguments)   /* In PARM_DECL.  */
-#define DECL_RESULT(NODE) ((NODE)->decl.result)
+#define DECL_ARG_TYPE(NODE) ((NODE)->decl.initial)     /* In PARM_DECL.  */
 #define DECL_INITIAL(NODE) ((NODE)->decl.initial)
 #define DECL_SOURCE_FILE(NODE) ((NODE)->decl.filename)
 #define DECL_SOURCE_LINE(NODE) ((NODE)->decl.linenum)
@@ -504,40 +504,51 @@ struct tree_type
 #define DECL_ALIGN(NODE) ((NODE)->decl.align)
 #define DECL_MODE(NODE) ((NODE)->decl.mode)
 #define DECL_RTL(NODE) ((NODE)->decl.rtl)
-#define DECL_BLOCK_SYMTAB_ADDRESS(NODE) ((NODE)->decl.block_symtab_address)
-#define DECL_SYMTAB_INDEX(NODE) ((NODE)->decl.block_symtab_address)
-#define DECL_SAVED_INSNS(NODE) ((NODE)->decl.saved_insns)
-#define DECL_FRAME_SIZE(NODE) ((NODE)->decl.frame_size)
+#define DECL_ASSEMBLER_NAME(NODE) ((NODE)->decl.assembler_name)
 #define DECL_LANG_SPECIFIC(NODE) ((NODE)->decl.lang_specific)
 
 struct tree_decl
 {
   char common[sizeof (struct tree_common)];
-  char *filename;
-  int linenum;
   union tree_node *size;
-  enum machine_mode mode;
+  enum machine_mode mode : 8;
   unsigned char size_unit;
   unsigned char align;
   unsigned char voffset_unit;
   union tree_node *name;
-  char *printable_name;
-  char *rtl_name;
   union tree_node *context;
   int offset;
   union tree_node *voffset;
-  union tree_node *arguments;
-  union tree_node *result;
   union tree_node *initial;
+  union tree_node *arguments;
   struct rtx_def *rtl;	/* acts as link to register transfer language
 				   (rtl) info */
+  /* Points to a structure whose details depend on the language in use.  */
+  struct lang_decl *lang_specific;
+
+  /* Unused by PARM_DECL.  */
+  char *filename;
+  int linenum;
+  char *assembler_name;
+};
+
+#define DECL_PRINT_NAME(NODE) ((NODE)->function_decl.print_name)
+#define DECL_RESULT(NODE) ((NODE)->function_decl.result)
+#define DECL_BLOCK_SYMTAB_ADDRESS(NODE) ((NODE)->function_decl.block_symtab_address)
+#define DECL_SAVED_INSNS(NODE) ((NODE)->function_decl.saved_insns)
+#define DECL_FRAME_SIZE(NODE) ((NODE)->function_decl.frame_size)
+
+struct tree_function_decl
+{
+  struct tree_decl ignore;
+
+  char *print_name;
+  union tree_node *result;
   int frame_size;		/* For FUNCTION_DECLs: size of stack frame */
   struct rtx_def *saved_insns;	/* For FUNCTION_DECLs: points to insn that
 				   constitutes its definition on the
 				   permanent obstack.  */
   int block_symtab_address;
-  /* Points to a structure whose details depend on the language in use.  */
-  struct lang_decl *lang_specific;
 };
 
 /* Define fields and accessors for nodes representing statements.
@@ -546,8 +557,7 @@ struct tree_decl
    in each contour) for the sake of outputting debugging info.
    Perhaps they will be used once again for other languages.  */
 
-/* For LABEL_STMT, GOTO_STMT, RETURN_STMT, LOOP_STMT,
-   COMPOUND_STMT, ASM_STMT.  */
+/* For LABEL_STMT, GOTO_STMT, RETURN_STMT, COMPOUND_STMT, ASM_STMT.  */
 #define STMT_SOURCE_LINE(NODE) ((NODE)->stmt.linenum)
 #define STMT_SOURCE_FILE(NODE) ((NODE)->stmt.filename)
 #define STMT_BODY(NODE) ((NODE)->stmt.body)
@@ -576,6 +586,19 @@ struct tree_if_stmt
   union tree_node *cond, *thenpart, *elsepart;
 };
 
+/* For LOOP_STMT.  */
+#define STMT_LOOP_VARS(NODE) ((NODE)->loop_stmt.vars)
+#define STMT_LOOP_COND(NODE) ((NODE)->loop_stmt.cond)
+#define STMT_LOOP_BODY(NODE) ((NODE)->loop_stmt.body)
+
+struct tree_loop_stmt
+{
+  char common[sizeof (struct tree_common)];
+  char *filename;
+  int linenum;
+  union tree_node *vars, *cond, *body;
+};
+
 /* For LET_STMT and WITH_STMT.  */
 
 /* #define STMT_SOURCE_LINE(NODE) */
@@ -585,6 +608,7 @@ struct tree_if_stmt
 #define STMT_SUPERCONTEXT(NODE) ((NODE)->bind_stmt.supercontext)
 #define STMT_BIND_SIZE(NODE) ((NODE)->bind_stmt.bind_size)
 #define STMT_TYPE_TAGS(NODE) ((NODE)->bind_stmt.type_tags)
+#define STMT_SUBBLOCKS(NODE) ((NODE)->bind_stmt.subblocks)
 
 struct tree_bind_stmt
 {
@@ -592,6 +616,7 @@ struct tree_bind_stmt
   char *filename;
   int linenum;
   union tree_node *body, *vars, *supercontext, *bind_size, *type_tags;
+  union tree_node *subblocks;
 };
 
 /* For CASE_STMT.  */
@@ -620,11 +645,14 @@ union tree_node
   struct tree_complex complex;
   struct tree_identifier identifier;
   struct tree_decl decl;
+  struct tree_function_decl function_decl;
   struct tree_type type;
   struct tree_list list;
+  struct tree_vec vec;
   struct tree_exp exp;
   struct tree_stmt stmt;
   struct tree_if_stmt if_stmt;
+  struct tree_loop_stmt loop_stmt;
   struct tree_bind_stmt bind_stmt;
   struct tree_case_stmt case_stmt;
 };
@@ -642,19 +670,21 @@ extern tree make_node ();
    for TREE_UID and TREE_PERMANENT.  (The copy is permanent
    iff nodes being made now are permanent.)  */
 
-extern tree copy_node (), copy_list ();
+extern tree copy_node ();
+
+/* Make a copy of a chain of TREE_LIST nodes.  */
+
+extern tree copy_list ();
+
+/* Make a TREE_VEC.  */
+
+extern tree make_tree_vec ();
 
 /* Return the (unique) IDENTIFIER_NODE node for a given name.
    The name is supplied as a char *.  */
 
 extern tree get_identifier ();
 
-/* Return a string for a decl's printable name.  */
-extern char *lang_printable_name ();
-
-/* Return a string for a decl's rtl name.  */
-extern char *lang_rtl_name ();
-
 /* Construct various types of nodes.  */
 
 extern tree build_int_2 ();
@@ -664,10 +694,12 @@ extern tree build_real_from_int_cst ();
 extern tree build_complex ();
 extern tree build_string ();
 extern tree build ();
-extern tree build_nt ();
-extern tree build_tree_list ();
+extern tree build_nt (), build_parse_node ();
+extern tree build_tree_list (), build_decl_list ();
 extern tree build_op_identifier ();
 extern tree build_decl ();
+extern tree build_if ();
+extern tree build_loop ();
 extern tree build_let ();
 
 /* Construct various nodes representing data types.  */
@@ -757,7 +789,7 @@ extern tree chainon ();
 /* Make a new TREE_LIST node from specified PURPOSE, VALUE and CHAIN.  */
 
 extern tree tree_cons (), perm_tree_cons (), temp_tree_cons ();
-extern tree saveable_tree_cons ();
+extern tree saveable_tree_cons (), decl_tree_cons ();
 
 /* Return the last tree node in a chain.  */
 
@@ -916,21 +948,25 @@ extern int current_function_calls_setjmp;
 /* Nonzero means all ..._TYPE nodes should be allocated permanently.  */
 
 extern int all_types_permanent;
+
+/* Pointer to function to compute the name to use to print a declaration.  */
+
+extern char *(*decl_printable_name) ();
 
 /* In stmt.c */
 
 extern tree expand_start_stmt_expr ();
 extern tree expand_end_stmt_expr ();
-extern void expand_expr_stmt(), clear_last_expr();
-extern void expand_label(), expand_goto(), expand_asm();
-extern void expand_start_cond(), expand_end_cond();
-extern void expand_start_else(), expand_end_else();
-extern void expand_start_loop(), expand_start_loop_continue_elsewhere();
-extern void expand_loop_continue_here();
-extern void expand_end_loop();
-extern int expand_continue_loop();
-extern int expand_exit_loop(), expand_exit_loop_if_false();
-extern int expand_exit_something();
+extern void expand_expr_stmt (), clear_last_expr ();
+extern void expand_label (), expand_goto (), expand_asm ();
+extern void expand_start_cond (), expand_end_cond ();
+extern void expand_start_else (), expand_end_else ();
+extern void expand_start_loop (), expand_start_loop_continue_elsewhere ();
+extern void expand_loop_continue_here ();
+extern void expand_end_loop ();
+extern int expand_continue_loop ();
+extern int expand_exit_loop (), expand_exit_loop_if_false ();
+extern int expand_exit_something ();
 
 extern void expand_start_delayed_expr ();
 extern tree expand_end_delayed_expr ();
